import { AttributeKey, CredentialDeploymentTransaction, CredentialDeploymentInfo, CryptographicParameters, IdentityInput, UnsignedCdiWithRandomness, UnsignedCredentialDeploymentInformation, VerifyKey, IpInfo, ArInfo, IdentityObjectV1, SignedCredentialDeploymentDetails, Network, CredentialPublicKeys } from './types';
import { TransactionExpiry } from './types/transactionExpiry';
import { AccountAddress } from './types/accountAddress';
import { AttributesKeys, CredentialDeploymentDetails, HexString } from '.';
/**
 * Create a credential deployment transaction, which is the transaction used
 * when deploying a new account.
 * @deprecated This function doesn't use allow supplying the randomness. {@link createCredentialTransactionV1 } or { @link createCredentialTransactionV1NoSeed } should be used instead.
 * @param identity the identity to create a credential for
 * @param cryptographicParameters the global cryptographic parameters from the chain
 * @param threshold the signature threshold for the credential, has to be less than number of public keys
 * @param publicKeys the public keys for the account
 * @param credentialIndex the index of the credential to create, has to be in sequence and unused
 * @param revealedAttributes the attributes about the account holder that should be revealed on chain
 * @param expiry the expiry of the transaction
 * @returns the details used in a credential deployment transaction
 */
export declare function createCredentialDeploymentTransaction(identity: IdentityInput, cryptographicParameters: CryptographicParameters, threshold: number, publicKeys: VerifyKey[], credentialIndex: number, revealedAttributes: AttributeKey[], expiry: TransactionExpiry): CredentialDeploymentTransaction;
/**
 * Create an unsigned credential for an existing account. This credential has to be signed by
 * the creator before it can be deployed on the existing account.
 * @param identity the identity to create a credential for
 * @param cryptographicParameters the global cryptographic parameters from the chain
 * @param threshold the signature threshold for the credential, has to be less than number of public keys
 * @param publicKeys the public keys for the credential
 * @param credentialIndex the index of the credential to create, has to be in sequence and unused
 * @param revealedAttributes the attributes about the account holder that should be revealed on chain
 * @param address the account address to associated the credential with
 */
export declare function createUnsignedCredentialForExistingAccount(identity: IdentityInput, cryptographicParameters: CryptographicParameters, threshold: number, publicKeys: VerifyKey[], credentialIndex: number, revealedAttributes: AttributeKey[], address: AccountAddress): UnsignedCdiWithRandomness;
/**
 * Combines the unsigned credential information and the signatures to the signed credential
 * deployment information. This is the information that the account owner needs to be able
 * to deploy the credential to their account.
 * @param unsignedCredentialInfo the unsigned credential information
 * @param signatures the signatures on the unsigned credential information
 * @returns signed credential deployment information, used in an update credentials transaction to deploy it
 */
export declare function buildSignedCredentialForExistingAccount(unsignedCredentialInfo: UnsignedCredentialDeploymentInformation, signatures: string[]): CredentialDeploymentInfo;
/**
 * Derives the account address from a credential id. This is the address of the
 * account that will be created by the credential deployment transaction containing
 * this credential id.
 * @param credId the credential id from a credential deployment transaction
 * @returns the account address
 */
export declare function getAccountAddress(credId: string): AccountAddress;
declare type CredentialInputCommon = {
    ipInfo: IpInfo;
    globalContext: CryptographicParameters;
    arsInfos: Record<string, ArInfo>;
    idObject: IdentityObjectV1;
    revealedAttributes: AttributeKey[];
    credNumber: number;
};
export declare type CredentialInput = CredentialInputCommon & {
    seedAsHex: string;
    net: Network;
    identityIndex: number;
};
/**
 * Creates a credential for a new account, using the version 1 algorithm, which uses a seed to generate keys and commitments.
 * @deprecated This function outputs the format used by the JSON-RPC client, createCredentialTransaction should be used instead.
 */
export declare function createCredentialV1(input: CredentialInput & {
    expiry: number;
}): SignedCredentialDeploymentDetails;
export declare type CredentialInputNoSeed = CredentialInputCommon & {
    idCredSec: HexString;
    prfKey: HexString;
    sigRetrievelRandomness: HexString;
    credentialPublicKeys: CredentialPublicKeys;
    attributeRandomness: Record<AttributesKeys, HexString>;
};
/**
 * Creates an unsigned credential for a new account, using the version 1 algorithm, which uses a seed to generate keys and commitments.
 */
export declare function createCredentialTransaction(input: CredentialInput, expiry: TransactionExpiry): CredentialDeploymentTransaction;
/**
 * Creates an unsigned credential for a new account, using the version 1 algorithm, but without requiring the seed to be provided directly.
 */
export declare function createCredentialTransactionNoSeed(input: CredentialInputNoSeed, expiry: TransactionExpiry): CredentialDeploymentTransaction;
export declare function signCredentialTransaction(credDeployment: CredentialDeploymentDetails, signingKey: HexString): Promise<HexString>;
export {};
