import { Big, BigSource } from 'big.js';
/**
 * Representation of a CCD amount.
 * The base unit of CCD is micro CCD, which is the representation
 * used on chain.
 */
export declare class CcdAmount {
    microCcdAmount: bigint;
    /**
     * Constructs a CcdAmount and checks that it is valid. It accepts a number, string, big, or bigint as parameter.
     * It can accept a string as parameter with either a comma or a dot as the decimal separator.
     *
     * @param microCcdAmount The amount of micro CCD as a number, string, big, or bigint.
     * @throws If an invalid micro CCD amount is passed, i.e. any value which is not an unsigned 64-bit integer
     */
    constructor(microCcdAmount: bigint | BigSource);
    /**
     * Returns the amount of micro CCD as a Big.
     *
     * @returns The amount of micro CCD as a Big
     */
    toMicroCcd(): Big;
    /**
     * Returns the amount of CCD as a Big.
     *
     * @returns The amount of CCD as a Big
     */
    toCcd(): Big;
    /**
     * Creates a CcdAmount from a number, string, big, or bigint.
     *
     * @param ccd The amount of micro CCD as a number, string, big or bigint.
     * @returns The CcdAmount object derived from the ccd input parameter
     * @throws If a number is passed with several decimal seperators
     * @throws If a negative amount of micro CCD is passed
     * @throws If the micro CCD passed is greater than what can be contained in a 64-bit integer
     */
    static fromCcd(ccd: BigSource | bigint): CcdAmount;
    /**
     * Converts an amount of CCD to micro CCD and asserts that the amount is a valid amount of CCD.
     *
     * @param ccd The amount of CCD as a number, string, big or bigint.
     * @returns The amount of micro CCD as a Big
     * @throws If a number is passed with several decimal seperators
     * @throws If a negative amount of micro CCD is passed
     * @throws If the micro CCD passed is greater than what can be contained in a 64-bit integer
     */
    static ccdToMicroCcd(ccd: BigSource | bigint): Big;
    /**
     * Converts an amount of micro CCD to CCD and asserts that the amount is a valid amount of CCD.
     *
     * @param microCcd The amount of micro CCD as a number, string, big or bigint.
     * @returns The amount of CCD as a Big
     * @throws If a number is passed with several decimal seperators
     * @throws If a negative amount of micro CCD is passed
     * @throws If the micro CCD passed is greater than what can be contained in a 64-bit integer
     */
    static microCcdToCcd(microCcd: BigSource | bigint): Big;
    toJSON(): string;
}
