import { Amount, AuthorizationsV0, AuthorizationsV1, Base58String, Duration, Energy, FinalizationCommitteeParameters, GasRewardsV0, GasRewardsV1, HexString, TimeoutParameters } from '..';
import type { IpInfo, ArInfo, VerifyKey, ExchangeRate, TransactionFeeDistribution, MintDistribution, MintRate, CommissionRates } from '../types';
declare type ChainUpdate<UpdateType, T> = {
    /** The type of the update */
    updateType: UpdateType;
    /** The parameters used for the update */
    update: T;
};
/** An update to mint distribution parameters */
export declare type MintDistributionUpdate = ChainUpdate<UpdateType.MintDistribution, MintDistribution>;
/** An update to the foundation account */
export declare type FoundationAccountUpdate = ChainUpdate<UpdateType.FoundationAccount, FoundationAccount>;
/** An update to election difficulty parameters */
export declare type ElectionDifficultyUpdate = ChainUpdate<UpdateType.ElectionDifficulty, ElectionDifficulty>;
/** An update to the euro per energy exchange rate */
export declare type EuroPerEnergyUpdate = ChainUpdate<UpdateType.EuroPerEnergy, ExchangeRate>;
/** An update to the micro CCD per euro exchange rate */
export declare type MicroGtuPerEuroUpdate = ChainUpdate<UpdateType.MicroGtuPerEuro, ExchangeRate>;
/** An update to transaction fee distribution parameters */
export declare type TransactionFeeDistributionUpdate = ChainUpdate<UpdateType.TransactionFeeDistribution, TransactionFeeDistribution>;
/** An update to gas reward parameters for protocol version 1-5 */
export declare type GasRewardsV0Update = ChainUpdate<UpdateType.GasRewards, GasRewardsV0>;
/** An update to gas reward parameters from protocol version 6 */
export declare type GasRewardsV1Update = ChainUpdate<UpdateType.GasRewardsCpv2, GasRewardsV1>;
/** An update to add an anonymity revoker */
export declare type AddAnonymityRevokerUpdate = ChainUpdate<UpdateType.AddAnonymityRevoker, AddAnonymityRevoker>;
/** An update to add an identity provider */
export declare type AddIdentityProviderUpdate = ChainUpdate<UpdateType.AddIdentityProvider, AddIdentityProvider>;
/** An update to staking cooldown parameters */
export declare type CooldownParametersUpdate = ChainUpdate<UpdateType.CooldownParameters, CooldownParameters>;
/** An update to time parameters */
export declare type TimeParametersUpdate = ChainUpdate<UpdateType.TimeParameters, TimeParameters>;
/** An update holding a protocol update */
export declare type ProtocolUpdate = ChainUpdate<UpdateType.Protocol, ProtocolUpdateDetails>;
/** An update to baker pool parameters */
export declare type PoolParametersUpdate = ChainUpdate<UpdateType.PoolParameters, PoolParameters>;
/** An update to baker stake threshold parameters */
export declare type BakerStakeThresholdUpdate = ChainUpdate<UpdateType.BakerStakeThreshold, BakerStakeThreshold>;
/** An update to timeout parameters, used from protocol version 6 */
export declare type TimeoutParametersUpdate = ChainUpdate<UpdateType.TimeoutParameters, TimeoutParameters>;
/** An update to mininum time between blocks, used from protocol version 6 */
export declare type MinBlockTimeUpdate = ChainUpdate<UpdateType.MinBlockTime, Duration>;
/** An update to maximum amount of energy per block, used from protocol version 6 */
export declare type BlockEnergyLimitUpdate = ChainUpdate<UpdateType.BlockEnergyLimit, Energy>;
/** An update to finalization committee parameters, used from protocol version 6 */
export declare type FinalizationCommitteeParametersUpdate = ChainUpdate<UpdateType.FinalizationCommitteeParameters, FinalizationCommitteeParameters>;
/** An update to level 1 key */
export declare type Level1Update = ChainUpdate<UpdateType.Level1, KeyUpdate>;
/** An update to root key */
export declare type RootUpdate = ChainUpdate<UpdateType.Root, KeyUpdate>;
/** A pending update to higher level keys */
export declare type PendingHigherLevelKeyUpdate = ChainUpdate<UpdateType.HigherLevelKeyUpdate, HigherLevelKeyUpdate>;
/** A pending update to authorization keys */
export declare type PendingAuthorizationKeysUpdate = ChainUpdate<UpdateType.AuthorizationKeysUpdate, AuthorizationKeysUpdate>;
/** A union of chain updates, barring key updates */
export declare type CommonUpdate = MicroGtuPerEuroUpdate | EuroPerEnergyUpdate | TransactionFeeDistributionUpdate | FoundationAccountUpdate | MintDistributionUpdate | ProtocolUpdate | GasRewardsV0Update | BakerStakeThresholdUpdate | ElectionDifficultyUpdate | AddAnonymityRevokerUpdate | AddIdentityProviderUpdate | CooldownParametersUpdate | PoolParametersUpdate | TimeParametersUpdate | GasRewardsV1Update | TimeoutParametersUpdate | MinBlockTimeUpdate | BlockEnergyLimitUpdate | FinalizationCommitteeParametersUpdate;
/** A union of chain updates */
export declare type UpdateInstructionPayload = CommonUpdate | RootUpdate | Level1Update;
/** A union of possible pending updates */
export declare type PendingUpdate = CommonUpdate | PendingHigherLevelKeyUpdate | PendingAuthorizationKeysUpdate;
/** Chain update types */
export declare enum UpdateType {
    Root = "root",
    Level1 = "level1",
    HigherLevelKeyUpdate = "higherLevelKeyUpdate",
    AuthorizationKeysUpdate = "AuthorizationKeysUpdate",
    Protocol = "protocol",
    ElectionDifficulty = "electionDifficulty",
    EuroPerEnergy = "euroPerEnergy",
    MicroGtuPerEuro = "microGtuPerEuro",
    FoundationAccount = "foundationAccount",
    MintDistribution = "mintDistribution",
    TransactionFeeDistribution = "transactionFeeDistribution",
    GasRewards = "gasRewards",
    PoolParameters = "poolParameters",
    AddAnonymityRevoker = "addAnonymityRevoker",
    AddIdentityProvider = "addIdentityProvider",
    CooldownParameters = "cooldownParameters",
    TimeParameters = "timeParameters",
    ProtocolUpdate = "protocolUpdate",
    BakerStakeThreshold = "bakerStakeThreshold",
    Emergency = "emergency",
    GasRewardsCpv2 = "gasRewardsCpv2",
    TimeoutParameters = "timeoutParameters",
    MinBlockTime = "minBlockTime",
    BlockEnergyLimit = "blockEnergyLimit",
    FinalizationCommitteeParameters = "finalizationCommitteeParameters"
}
export declare type KeyUpdate = HigherLevelKeyUpdate | AuthorizationKeysUpdate;
export interface Fraction {
    numerator: bigint;
    denominator: bigint;
}
export declare type AddIdentityProvider = IpInfo;
export declare type AddAnonymityRevoker = ArInfo;
export interface FoundationAccount {
    address: Base58String;
}
export interface ProtocolUpdateDetails {
    message: string;
    specificationUrl: string;
    specificationHash: HexString;
    specificationAuxiliaryData: string;
}
export interface BakerStakeThreshold {
    threshold: bigint;
}
export interface ElectionDifficulty {
    electionDifficulty: number;
}
export interface TimeParameters {
    rewardPeriodLength: bigint;
    mintRatePerPayday: MintRate;
}
export interface CooldownParameters {
    poolOwnerCooldown: bigint;
    delegatorCooldown: bigint;
}
export interface CommissionRange {
    min: number;
    max: number;
}
export interface CommissionRanges {
    finalizationRewardCommission: CommissionRange;
    bakingRewardCommission: CommissionRange;
    transactionFeeCommission: CommissionRange;
}
export interface PoolParameters {
    passiveCommissions: CommissionRates;
    commissionBounds: CommissionRanges;
    minimumEquityCapital: Amount;
    capitalBound: number;
    leverageBound: Fraction;
}
export declare enum KeyUpdateEntryStatus {
    Added = 0,
    Removed = 1,
    Unchanged = 2
}
export interface KeyWithStatus {
    key: VerifyKey;
    status: KeyUpdateEntryStatus;
}
export declare enum HigherLevelKeyUpdateType {
    RootKeysUpdate = "rootKeysUpdate",
    Level1KeysUpdate = "level1KeysUpdate"
}
export interface HigherLevelKeyUpdate {
    typeOfUpdate: HigherLevelKeyUpdateType;
    updateKeys: VerifyKey[];
    threshold: number;
}
export declare enum AuthorizationKeysUpdateType {
    Level2KeysUpdate = "level2KeysUpdate",
    Level2KeysUpdateV1 = "level2KeysUpdateV1"
}
export declare type AuthorizationKeysUpdate = {
    typeOfUpdate: AuthorizationKeysUpdateType.Level2KeysUpdate;
    updatePayload: AuthorizationsV0;
} | {
    typeOfUpdate: AuthorizationKeysUpdateType.Level2KeysUpdateV1;
    updatePayload: AuthorizationsV1;
};
export declare type UpdateInstructionHeader = {
    sequenceNumber: bigint;
    effectiveTime: bigint;
    timeout: bigint;
};
export declare type UpdateInstruction = {
    header: UpdateInstructionHeader;
    payload: HexString;
};
export {};
