import type { OpenStatusText, ContractAddress, ReleaseSchedule, ContractVersion, Address, Base58String, ModuleRef, HexString, EventDelegationTarget, Amount, BakerId, DelegatorId } from '../types';
import type { UpdateInstructionPayload } from './chainUpdate';
export declare enum TransactionEventTag {
    ModuleDeployed = "ModuleDeployed",
    ContractInitialized = "ContractInitialized",
    AccountCreated = "AccountCreated",
    CredentialDeployed = "CredentialDeployed",
    BakerAdded = "BakerAdded",
    BakerRemoved = "BakerRemoved",
    BakerStakeIncreased = "BakerStakeIncreased",
    BakerStakeDecreased = "BakerStakeDecreased",
    BakerSetRestakeEarnings = "BakerSetRestakeEarnings",
    BakerKeysUpdated = "BakerKeysUpdated",
    CredentialKeysUpdated = "CredentialKeysUpdated",
    NewEncryptedAmount = "NewEncryptedAmount",
    EncryptedAmountsRemoved = "EncryptedAmountsRemoved",
    AmountAddedByDecryption = "AmountAddedByDecryption",
    EncryptedSelfAmountAdded = "EncryptedSelfAmountAdded",
    UpdateEnqueued = "UpdateEnqueued",
    TransferredWithSchedule = "TransferredWithSchedule",
    CredentialsUpdated = "CredentialsUpdated",
    DataRegistered = "DataRegistered",
    BakerSetOpenStatus = "BakerSetOpenStatus",
    BakerSetMetadataURL = "BakerSetMetadataURL",
    BakerSetTransactionFeeCommission = "BakerSetTransactionFeeCommission",
    BakerSetBakingRewardCommission = "BakerSetBakingRewardCommission",
    BakerSetFinalizationRewardCommission = "BakerSetFinalizationRewardCommission",
    DelegationStakeIncreased = "DelegationStakeIncreased",
    DelegationStakeDecreased = "DelegationStakeDecreased",
    DelegationSetRestakeEarnings = "DelegationSetRestakeEarnings",
    DelegationSetDelegationTarget = "DelegationSetDelegationTarget",
    DelegationAdded = "DelegationAdded",
    DelegationRemoved = "DelegationRemoved",
    TransferMemo = "TransferMemo",
    Transferred = "Transferred",
    Interrupted = "Interrupted",
    Resumed = "Resumed",
    Updated = "Updated",
    Upgraded = "Upgraded"
}
export declare type TransactionEvent = TransferredEvent | UpdatedEvent | ResumedEvent | InterruptedEvent | UpgradedEvent | MemoEvent | TransferredWithScheduleEvent | AccountCreatedEvent | AmountAddedByDecryptionEvent | EncryptedSelfAmountAddedEvent | NewEncryptedAmountEvent | EncryptedAmountsRemovedEvent | DataRegisteredEvent | ContractInitializedEvent | ModuleDeployedEvent | CredentialDeployedEvent | CredentialKeysUpdatedEvent | CredentialsUpdatedEvent | DelegatorEvent | DelegationSetDelegationTargetEvent | DelegationSetRestakeEarningsEvent | DelegationStakeChangedEvent | BakerAddedEvent | BakerRemovedEvent | BakerStakeChangedEvent | BakerSetRestakeEarningsEvent | BakerKeysUpdatedEvent | BakerSetOpenStatusEvent | BakerSetMetadataURLEvent | BakerSetFinalizationRewardCommissionEvent | BakerSetBakingRewardCommissionEvent | BakerSetTransactionFeeCommissionEvent | UpdateEnqueuedEvent;
export interface InterruptedEvent {
    tag: TransactionEventTag.Interrupted;
    address: ContractAddress;
    events: HexString[];
}
export interface ResumedEvent {
    tag: TransactionEventTag.Resumed;
    address: ContractAddress;
    success: boolean;
}
export interface UpdatedEvent {
    tag: TransactionEventTag.Updated;
    address: ContractAddress;
    instigator: Address;
    amount: Amount;
    contractVersion: ContractVersion;
    message: HexString;
    receiveName: string;
    events: HexString[];
}
export interface UpgradedEvent {
    tag: TransactionEventTag.Upgraded;
    address: ContractAddress;
    from: ModuleRef;
    to: ModuleRef;
}
export interface DataRegisteredEvent {
    tag: TransactionEventTag.DataRegistered;
    data: HexString;
}
export interface ContractInitializedEvent {
    tag: TransactionEventTag.ContractInitialized;
    address: ContractAddress;
    amount: Amount;
    initName: string;
    events: HexString[];
    contractVersion: ContractVersion;
    ref: ModuleRef;
}
export interface ModuleDeployedEvent {
    tag: TransactionEventTag.ModuleDeployed;
    contents: ModuleRef;
}
export interface AccountTransferredEvent {
    tag: TransactionEventTag.Transferred;
    amount: Amount;
    to: Base58String;
}
export interface TransferredEvent {
    tag: TransactionEventTag.Transferred;
    amount: Amount;
    to: Address;
    from?: Address;
}
export interface TransferredWithScheduleEvent {
    tag: TransactionEventTag.TransferredWithSchedule;
    to: Base58String;
    from?: Base58String;
    amount: ReleaseSchedule[];
}
export interface MemoEvent {
    tag: TransactionEventTag.TransferMemo;
    memo: HexString;
}
export interface AccountCreatedEvent {
    tag: TransactionEventTag.AccountCreated;
    account: Base58String;
}
export interface AmountAddedByDecryptionEvent {
    tag: TransactionEventTag.AmountAddedByDecryption;
    account: Base58String;
    amount: Amount;
}
export interface EncryptedSelfAmountAddedEvent {
    tag: TransactionEventTag.EncryptedSelfAmountAdded;
    account: Base58String;
    amount: Amount;
    newAmount: string;
}
export interface EncryptedAmountsRemovedEvent {
    tag: TransactionEventTag.EncryptedAmountsRemoved;
    account: Base58String;
    inputAmount: HexString;
    newAmount: HexString;
    upToIndex: number;
}
export interface NewEncryptedAmountEvent {
    tag: TransactionEventTag.NewEncryptedAmount;
    account: Base58String;
    newIndex: number;
    encryptedAmount: HexString;
}
export interface CredentialDeployedEvent {
    tag: TransactionEventTag.CredentialDeployed;
    regid: HexString;
    account: Base58String;
}
export interface CredentialKeysUpdatedEvent {
    tag: TransactionEventTag.CredentialKeysUpdated;
    credId: HexString;
}
export interface CredentialsUpdatedEvent {
    tag: TransactionEventTag.CredentialsUpdated;
    account: Base58String;
    newCredIds: HexString[];
    removedCredIds: HexString[];
    newThreshold: number;
}
export interface DelegatorEvent {
    tag: TransactionEventTag.DelegationAdded | TransactionEventTag.DelegationRemoved;
    delegatorId: DelegatorId;
    account: Base58String;
}
export interface DelegationSetDelegationTargetEvent {
    tag: TransactionEventTag.DelegationSetDelegationTarget;
    delegatorId: DelegatorId;
    account: Base58String;
    delegationTarget: EventDelegationTarget;
}
export interface DelegationSetRestakeEarningsEvent {
    tag: TransactionEventTag.DelegationSetRestakeEarnings;
    delegatorId: DelegatorId;
    account: Base58String;
    restakeEarnings: boolean;
}
export interface DelegationStakeChangedEvent {
    tag: TransactionEventTag.DelegationStakeDecreased | TransactionEventTag.DelegationStakeIncreased;
    delegatorId: DelegatorId;
    account: Base58String;
    newStake: bigint;
}
export interface BakerAddedEvent {
    tag: TransactionEventTag.BakerAdded;
    bakerId: BakerId;
    account: string;
    signKey: string;
    electionKey: string;
    aggregationKey: string;
    stake: bigint;
    restakeEarnings: boolean;
}
export interface BakerRemovedEvent {
    tag: TransactionEventTag.BakerRemoved;
    bakerId: BakerId;
    account: Base58String;
}
export interface BakerStakeChangedEvent {
    tag: TransactionEventTag.BakerStakeIncreased | TransactionEventTag.BakerStakeDecreased;
    bakerId: BakerId;
    account: Base58String;
    newStake: bigint;
}
export interface BakerSetRestakeEarningsEvent {
    tag: TransactionEventTag.BakerSetRestakeEarnings;
    bakerId: BakerId;
    account: Base58String;
    restakeEarnings: boolean;
}
export interface BakerKeysUpdatedEvent {
    tag: TransactionEventTag.BakerKeysUpdated;
    bakerId: BakerId;
    account: Base58String;
    signKey: HexString;
    electionKey: HexString;
    aggregationKey: HexString;
}
export interface BakerSetOpenStatusEvent {
    tag: TransactionEventTag.BakerSetOpenStatus;
    bakerId: BakerId;
    account: Base58String;
    openStatus: OpenStatusText;
}
export interface BakerSetMetadataURLEvent {
    tag: TransactionEventTag.BakerSetMetadataURL;
    bakerId: BakerId;
    account: Base58String;
    metadataURL: string;
}
export interface BakerSetFinalizationRewardCommissionEvent {
    tag: TransactionEventTag.BakerSetFinalizationRewardCommission;
    bakerId: BakerId;
    account: Base58String;
    finalizationRewardCommission: number;
}
export interface BakerSetBakingRewardCommissionEvent {
    tag: TransactionEventTag.BakerSetBakingRewardCommission;
    bakerId: BakerId;
    account: Base58String;
    bakingRewardCommission: number;
}
export interface BakerSetTransactionFeeCommissionEvent {
    tag: TransactionEventTag.BakerSetTransactionFeeCommission;
    bakerId: BakerId;
    account: Base58String;
    transactionFeeCommission: number;
}
export interface UpdateEnqueuedEvent {
    tag: TransactionEventTag.UpdateEnqueued;
    effectiveTime: number;
    payload: UpdateInstructionPayload;
}
export declare type ContractTraceEvent = ResumedEvent | InterruptedEvent | UpdatedEvent | UpgradedEvent | TransferredEvent;
export declare type BakerEvent = BakerSetTransactionFeeCommissionEvent | BakerSetBakingRewardCommissionEvent | BakerSetFinalizationRewardCommissionEvent | BakerSetMetadataURLEvent | BakerSetOpenStatusEvent | BakerSetRestakeEarningsEvent | BakerStakeChangedEvent | BakerAddedEvent | BakerRemovedEvent | BakerKeysUpdatedEvent;
export declare type DelegationEvent = DelegatorEvent | DelegationSetDelegationTargetEvent | DelegationSetRestakeEarningsEvent | DelegationStakeChangedEvent;
